#!/bin/bash
#
##################
##### LiHAS ######
##################
#
# Paket: aptchk4u
#
# /usr/bin/aptchk4u.sh: Main Script
#
# Autoren: Thomas Hoth (Mail, Tmpfile, Ratschlaege),
#          Daniel Philipp (vserver Erweiterungen, Rewrite, Funktionen)
#          Johannes Schöpfer, umbauen um aptitude anstatt apt-get zu ebnutzen
#
# Beschreibung: Skript zur Ueberpruefung von vorhandenen Updates fuer Debian;
#               ueberprueft Hostsystem und virtuelle Server mit
#               anschliessendem Versenden einer E-Mail bei notwendigen
#               Updates
# letzte Aenderung: 2008-10-17-jsc


#### VARIABLEN

HOSTNAME="$(hostname)"

# Programme, die nicht im Paket "coreutils" enthalten sind
AWK=awk
MUTT=mutt
SED=sed


# auf Vserver bezogene Variablen
VS_DIR=/var/lib/vservers
VS_EXE=vserver

# temporaere Datei (beinhaltet Namen der Upgrade-Pakete)
TMPFILE=$(mktemp /tmp/aptchk4u-tmp.XXXXXX)

CONFIGFILE="/etc/aptchk4u/aptchk4u.rc"
MUTTCONFIG="/etc/aptchk4u/muttrc"

### FUNKTIONEN:

# vserver Hauptfunktion: diese Funktion fragt den Status der aktiven,
# virtuellen Server ab und aktualisiert dann mittels der 1.
# Unterfunktion die Paketlisten der einzelnen virtuellen Server;
# mittels der 2. Unterfunktion wird eine Liste der benoetigten Pakete
# erstellt und anschliessend eine E-Mail je virtuellem Server verschickt

function check_vs_run() {
  cd $VS_DIR
    ls | while read a; do
      vserver $a running 2> /dev/null
      if [ "$?" == 0 ]; then
        vs_agud $a
        check_vs_pack $a
      fi
    done
 }


function aptversion() {
  if $vserver [ -e /etc/debian_version ]; then
  # Check if sarge or etch, for sarge use apt-get
    debianv=$($vserver cat /etc/debian_version | cut -d'.' -f1| cut -d'/' -f1)
    [ "$debianv" = etch ] && debianv=4
    [ "$debianv" = lenny ] && debianv=5
    if [ "$debianv" -gt 3 -o "$debianv" = etch ]
    then
       APT_G="aptitude -o Aptitude::CmdLine::Ignore-Trust-Violations=true"
    else
       APT_G=apt-get
    fi
    apt_p=$($vserver which ${APT_G%% *})
    $vserver [ -x $apt_p ] || APT_G=false
  else
     debianv=0
     APT_G=false
  fi
}
  
# vserver 1. Unterfunktion: aktualisiert die Paketlisten der einzelnen
# virtuellen Server
  
function vs_agud() {
  vserver="$VS_EXE $a exec" 
  aptversion
  $vserver $APT_G update > /dev/null
}

# vserver 2. Unterfunktion: erstellt Liste der benoetigten Pakete und haengt
# diese an eine temporaere Datei an

function check_vs_pack() {
  VS_HOSTNAME=$($VS_EXE $a exec cat /etc/hostname)
  if [ "$APT_G" == false ]; then
    echo -e "vserver $VS_HOSTNAME:   aptitude not found or not executable\n" >> $TMPFILE
  else
    PACKAGES=$($VS_EXE $a exec $APT_G -q -y -s dist-upgrade | grep ^\ )
    if [ -n "$PACKAGES" ] ; then
      echo -e "vserver $VS_HOSTNAME:   $PACKAGES \n" >> $TMPFILE
    fi
  fi
  if [ "$debianv" == 0 ]; then
    echo -e "    missing /etc/debian_version\n" >> $TMPFILE
  fi
}

# Host-Funktion: aktualisiert die Paketlisten des Hosts, erstellt eine Liste
# der benoetigten Pakete und haengt diese an eine temporaere Datei an

function check_host_pack() {
vserver=
aptversion
  $APT_G update > /dev/null
  PACKAGES=$($APT_G -q -y -s dist-upgrade | grep ^\ )
  if [ -n "$PACKAGES" ] ; then
    echo -e "$HOSTNAME:   $PACKAGES \n" >> $TMPFILE
  fi
}

# Mailfunktion: verschickt den Inhalt der temporaeren Datei, wenn Inhalt > 0

function mail_tmpfile() {
  if [ -n "$(cat $TMPFILE)" ]; then
    cat $TMPFILE | $MUTT -F $MUTTCONFIG -s "$name_of_company: Update on host \"$HOSTNAME\" needed" $email_address
  else
    echo "temporary file not found or zero content" > /dev/null
  fi
}

###


### MAIN:

. $CONFIGFILE

# 1. Parameter ist E-Mail-Adresse, wenn vorhanden;
# ansonsten E-Mail-Adresse aus Konfigurationsdatei
if [ ! -z "$1" ] ; then
  email_address="$1"
fi

check_host_pack

# ueberbruefe Existenz des vserver Verzeichnissses unter /
# und fuehre erst dann die vserver Hauptfunktion aus

if [ -d "$VS_DIR" ]; then
  check_vs_run
else
  echo "vservers directory not found" > /dev/null
fi

mail_tmpfile

rm $TMPFILE
